﻿using System;
using System.Net.Http;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Polly;
using Polly.CircuitBreaker;
using Polly.Retry;

namespace PollyCircuitBreakerLocal.Controllers
{
    [Produces("application/json")]
    [Route("[controller]")]
    public class CatalogController : Controller
    {
        private readonly HttpClient _httpClient;

        private readonly CircuitBreakerPolicy<HttpResponseMessage> _breakerPolicy;
        public CatalogController(HttpClient httpClient, CircuitBreakerPolicy<HttpResponseMessage> breakerPolicy)
        {
            _breakerPolicy = breakerPolicy;
            _httpClient = httpClient;
        }

        [HttpGet("{catalogId}/inventory")]
        public async Task<IActionResult> Get(int catalogId)
        {
            string inventoryEndpoint = $"inventory/{catalogId}";

            HttpResponseMessage inventoryResponse = await _breakerPolicy.ExecuteAsync(
                     () => _httpClient.GetAsync(inventoryEndpoint));

            if (inventoryResponse.IsSuccessStatusCode)
            {
                int itemsAvailable = await inventoryResponse.Content.ReadAsAsync<int>();
                Console.WriteLine($"\tInventory Response: {inventoryResponse.StatusCode} {itemsAvailable}");
                return Ok(itemsAvailable);
            }

            Console.WriteLine($"\tInventory Response: {inventoryResponse.ReasonPhrase}");
            return StatusCode((int)inventoryResponse.StatusCode, inventoryResponse.Content.ReadAsStringAsync());
        }

        [HttpGet("{catalogId}/price")]
        public async Task<IActionResult> GetPrice(int catalogId)
        {
            string requestEndpoint = $"price/{catalogId}";

            HttpResponseMessage pricingResponse = await _breakerPolicy.ExecuteAsync(
                    () => _httpClient.GetAsync(requestEndpoint));

            if (pricingResponse.IsSuccessStatusCode)
            {
                double price = await pricingResponse.Content.ReadAsAsync<double>();
                Console.WriteLine($"\tInventory Response: {pricingResponse.StatusCode} {price}");
                return Ok($"{price}");
            }

            Console.WriteLine($"\tPricing Response: {pricingResponse.ReasonPhrase}");
            return StatusCode((int)pricingResponse.StatusCode, pricingResponse.Content.ReadAsStringAsync());
        }

    }
}
